﻿// © Copyright 2011 Agilent Technologies, Inc.

using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace Agilent.Vee.EvoExample.ExpressionGraph
{
    /// <summary>
    /// Interaction logic for ExpressionGraphControl.
    /// </summary>
    public sealed partial class GraphControl : UserControl
    {
        Double[] m_xCoordinate;
        Double[] m_yCoordinate;
        EventHandler m_changesEventHandler;
        EventHandler m_terminalEventHandler;
        // Canvas size ActualWidth changes during execution due to hide column.
        // We need this so that the graph size doesn't change before/after execution.
        Double m_termColumnWidth; 
        
        public GraphControl()
        {
            InitializeComponent();
        }

        #region Events
        /// <summary>
        /// Occurs when there is changes on the text boxes value. 
        /// </summary>
        public event EventHandler ContentChanged
        {
            add
            {
                m_changesEventHandler += value;
            }
            remove
            {
                m_changesEventHandler -= value;
            }
        }

        /// <summary>
        /// Occurs when there is changes on graph control.
        /// </summary>
        public event EventHandler TerminalChanged
        {
            add
            {
                m_terminalEventHandler += value;
            }
            remove
            {
                m_terminalEventHandler -= value;
            }
        }

        #endregion // Events

        #region Properties

        /// <summary>
        /// Gets/Sets Y expression.
        /// </summary>
        public String YExpression
        {
            get
            {
                return m_expressionTextBox.Text;
            }
            set
            {
                m_expressionTextBox.Text = value;
            }
            
        }

        /// <summary>
        /// Gets/Sets X starting point.
        /// </summary>
        public Double XFrom
        {
            get
            {
                Double from;
                if (!Double.TryParse(m_fromTextBox.Text, out from))
                {
                    MessageBox.Show("X's From must be of Double value. It will be reset to -5.", "X's From");
                    m_fromTextBox.Text = "-5";
                    return -5;
                }

                return from;
            }
            set
            {
                m_fromTextBox.Text = value.ToString();
            }
        }

        /// <summary>
        /// Gets/Sets X ending point.
        /// </summary>
        public Double XTo
        {
            get
            {
                Double to;
                if (!Double.TryParse(m_toTextBox.Text, out to))
                {
                    MessageBox.Show("X's To must be of Double value. It will be reset to 5.", "X's To");
                    m_toTextBox.Text = "5";
                    return 5;
                }

                return to;
            }
            set
            {
                m_toTextBox.Text = value.ToString();
            }
        }

        /// <summary>
        /// Gets/Sets the waveform data size. In other words, the number of points in
        /// the generated waveform.
        /// </summary>
        public Int32 DataSize
        {
            get
            {
                Int32 dataSize;
                if (!Int32.TryParse(m_dataSizeTextBox.Text, out dataSize))
                {
                    // For demo purposes, MessageBox is used.
                    // In automation cases, should have throw Exception instead.
                    MessageBox.Show("Data Size must be of Integer value. It will be reset to 2000.", "Data Size");
                    m_dataSizeTextBox.Text = "2000";
                    return 2000;
                }

                return dataSize;
            }
            set
            {
                m_dataSizeTextBox.Text = value.ToString();
            }
        }

        /// <summary>
        /// Gets the generated data point's X-coordinates.
        /// </summary>
        public Double[] XCoordinate
        {
            get
            {
                if (m_xCoordinate != null)
                {
                    return m_xCoordinate;
                }
                return new Double[0];
            }
        }

        /// <summary>
        /// Gets the generated data point's Y-coordinates.
        /// </summary>
        public Double[] YCoordinate
        {
            get
            {
                if (m_yCoordinate != null)
                {
                    return m_yCoordinate;
                }
                return new Double[0];
            }
        }

        /// <summary>
        /// Gets/Sets if Y-Expression terminal is added.
        /// </summary>
        public Boolean YTermAdded
        {
            get
            {
                return m_yCheckBox.IsChecked.Value;
            }
            set
            {
                m_yCheckBox.IsChecked = true;
            }
        }

        /// <summary>
        /// Gets/Sets if X's From and To terminals are added.
        /// </summary>
        public Boolean XTermAdded
        {
            get
            {
                return m_xCheckBox.IsChecked.Value;
            }
            set
            {
                m_xCheckBox.IsChecked = true;
            }
        }

        /// <summary>
        /// Gets/Sets if Data Size terminal is added.
        /// </summary>
        public Boolean DataSizeTermAdded
        {
            get
            {
                return m_dataSizeCheckBox.IsChecked.Value;
            }
            set
            {
                m_dataSizeCheckBox.IsChecked = true;
            }
        }

        #endregion // Properties

        #region Method

        /// <summary>
        /// Generate waveform data.
        /// </summary>
        internal void GenerateWaveform()
        {
            TreeParser parser = new TreeParser("(" + m_expressionTextBox.Text + ")");

            Double to = XTo;
            Double from = XFrom;
            Int32 numberOfPoints = DataSize;
            Double canvasActualHeight = m_canvas.ActualHeight;
            Double canvasActualWidth = m_canvas.ActualWidth;
            Double scaleX;
            Double scaleY;
            Point[] points = new Point[numberOfPoints];
            m_xCoordinate = new Double[numberOfPoints];
            m_yCoordinate = new Double[numberOfPoints];

            Double deltaX = to - from;
            Double step = deltaX / (numberOfPoints - 1);
            Double max = Double.MinValue;
            Double min = Double.MaxValue;

            // Draw on 90% of the canvas size.
            // Set the starting point accordingly.
            // The graph is showing the shape, so its point X value 
            // doesn't need to be actual X value.
            // Similarly, its Y value doesn't need to be actual Y value.
            scaleX = (canvasActualWidth - m_termColumnWidth) * 0.9 / numberOfPoints;
            for (Int32 i = 0; i < numberOfPoints; i++)
            {
                Double value;
                points[i].X = i * scaleX;
                m_xCoordinate[i] = parser.XValue = i * step + from;
                value = m_yCoordinate[i] = parser.Value;

                if (value > max)
                    max = value;

                if (value < min)
                    min = value;
            }

            scaleY = canvasActualHeight * 0.9/ (max - min);

            // Set the graph starting point to somehwere near top left.
            Double offsetX = (canvasActualWidth - m_termColumnWidth) * 0.05;
            Double offsetY = canvasActualHeight * 0.05;

            Canvas.SetLeft(m_poly, offsetX);
            Canvas.SetTop(m_poly, offsetY);

            for (Int32 i = 0; i < numberOfPoints; i++)
            {
                // We need to shift the data point to fit to 
                // WPF data point calculations.
                points[i].Y = (max - m_yCoordinate[i]) * scaleY;
            }            

            m_poly.Points = new PointCollection(points);
        }

        /// <summary>
        /// Hide Use Term. column in UI display.
        /// </summary>
        internal void HideTerminalColumn()
        {
            m_termColumnWidth = m_termColumn.ActualWidth;
            m_termColumn.Width = new GridLength(0);            
        }

        /// <summary>
        /// Show Use Term. column in UI display.
        /// </summary>
        internal void ShowTerminalColumn()
        {
            m_termColumn.Width = new GridLength(0.7, GridUnitType.Star);
            m_termColumnWidth = 0;
        }

        /// <summary>
        /// Check X's To & From value, Data Size value if they are of correct types and in correct format.
        /// </summary>
        internal void CheckValue()
        {
            CheckXValue();
            CheckDataSizeValue();
        }

        private void CheckDataSizeValue()
        {
            String message;
            Int32 number;

            // Parse data size value.
            if (Int32.TryParse(m_dataSizeTextBox.Text, out number))
            {
                // A simple created rule for this example.
                if (number < 10)
                {
                    message = "Number of Points must be at least 10.";
                    throw new ApplicationException(message);
                }
            }
            else
            {
                message = "Number of Points must be of Integer value";
                throw new ApplicationException(message);
            }
        }

        private void CheckXValue()
        {
            Double from, to;
            String message;

            if (Double.TryParse(m_fromTextBox.Text, out from)
                && Double.TryParse(m_toTextBox.Text, out to))
            {
                if (from > to)
                {
                    message = "X's From value must be smaller than X's To value.";
                    throw new ApplicationException(message);
                }
            }
            else
            {
                message = "X's From and To value must be of Double value.";
                throw new ApplicationException(message); 
            }
        }

        private void AddYTerminal()
        {
            m_expressionTextBox.IsEnabled = false;
            FireTerminalEvent();
        }

        private void RemoveYTerminal()
        {
            m_expressionTextBox.IsEnabled = true;
            FireTerminalEvent();
        }

        private void AddXTerminal()
        {
            m_fromTextBox.IsEnabled = false;
            m_toTextBox.IsEnabled = false;
            FireTerminalEvent();
        }

        private void RemoveXTerminal()
        {
            m_fromTextBox.IsEnabled = true;
            m_toTextBox.IsEnabled = true;
            FireTerminalEvent();
        }

        private void AddSizeTerminal()
        {
            m_dataSizeTextBox.IsEnabled = false;
            FireTerminalEvent();
        }

        private void RemoveSizeTerminal()
        {
            m_dataSizeTextBox.IsEnabled = true;
            FireTerminalEvent();
        }

        private void FireChangesEvent()
        {
            if (m_changesEventHandler != null)
            {
                m_changesEventHandler.Invoke(this, EventArgs.Empty);
            }
        }

        private void FireTerminalEvent()
        {
            if (m_terminalEventHandler != null)
            {
                m_terminalEventHandler.Invoke(this, EventArgs.Empty);
            }
        }
                
        private void m_yCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            AddYTerminal();
        }

        private void m_yCheckBox_Unchecked(object sender, RoutedEventArgs e)
        {
            RemoveYTerminal();
        }

        private void m_xCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            AddXTerminal();
        }

        private void m_xCheckBox_Unchecked(object sender, RoutedEventArgs e)
        {
            RemoveXTerminal();
        }

        private void m_sizeCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            AddSizeTerminal();
        }

        private void m_sizeCheckBox_Unchecked(object sender, RoutedEventArgs e)
        {
            RemoveSizeTerminal();
        }

        private void Value_TextChanged(object sender, TextChangedEventArgs e)
        {
            FireChangesEvent();
        }

        private void m_previewButton_Click(object sender, RoutedEventArgs e)
        {
            string msgCaption = String.Empty;
            try
            {
                msgCaption = "X's From & To";
                CheckXValue();
                msgCaption = "Number of Points";
                CheckDataSizeValue();
                GenerateWaveform();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, msgCaption);
            }
        }

        private void m_fromTextBox_LostFocus(object sender, RoutedEventArgs e)
        {
            Double from;
            if (!Double.TryParse(m_fromTextBox.Text, out from))
            {
                MessageBox.Show("X's From must be of Double value.", "X's From");
            }
        }

        private void m_toTextBox_LostFocus(object sender, RoutedEventArgs e)
        {
            Double to;
            if (!Double.TryParse(m_fromTextBox.Text, out to))
            {
                MessageBox.Show("X's To must be of Double value.", "X's To");
            }
        }

        private void m_dataSizeTextBox_LostFocus(object sender, RoutedEventArgs e)
        {
            Int32 size;
            if (!Int32.TryParse(m_dataSizeTextBox.Text, out size))
            {
                MessageBox.Show("Data Size must be of Integer value.", "Data Size");
            }
        }

        #endregion // Method
    }
}
